/*
 *  TcPreferences.cpp
 *  ToyCars
 *
 *  Created by Ruben Henner Zilibowitz on 16/12/05.
 *  Copyright 2005 Ruben Henner Zilibowitz. All rights reserved.
 *
 */

#include "TcPreferences.h"
#include "tinyxml.h"
#include "ScException.h"
#include "Menu.h"
#include <cstdio>
#include <cstring>

void DrawSplash();

TcPreferences::TcPreferences(const char *inFilename)
	: modifiedScreenPrefs(false)
{
	filename = strdup(inFilename);
	TiXmlDocument *xmlDoc = new TiXmlDocument(filename);
	if (!xmlDoc->LoadFile())
	{
        // create default preferences
        
		defaultPrefs();
        savePreferences(filename);
    }
    else
    {
		try {
			TiXmlHandle docHandle(xmlDoc);
			
			// screen element
			
			TiXmlElement *xScreen = docHandle.FirstChildElement("preferences").FirstChild("screen").Element();
			if (!xScreen)
				ScThrowErr("xml preferences could not find screen element");
			if (xScreen->QueryIntAttribute("width", &screenwidth) != TIXML_SUCCESS)
				ScThrowErr("xml preferences screen element could not find width attribute");
			if (xScreen->QueryIntAttribute("height", &screenheight) != TIXML_SUCCESS)
				ScThrowErr("xml preferences screen element could not find height attribute");
			
			const char *fullscreen_attr = xScreen->Attribute("fullscreen");
			if (fullscreen_attr == NULL)
				ScThrowErr("xml preferences screen element could not find fullscreen attribute");
			if (strcmp(fullscreen_attr, "yes") == 0)
				fullscreen = 1;
			else if (strcmp(fullscreen_attr, "no") == 0)
				fullscreen = 0;
			else
				ScThrowErr("xml preferences fullscreen attribute has unknown value");
			
			// graphics elements
			
			TiXmlElement *xGraphics = docHandle.FirstChildElement("preferences").FirstChild("graphics").Element();
			if (!xGraphics)
				ScThrowErr("xml preferences could not find graphics element");
			if (xGraphics->QueryIntAttribute("maxParticles", &maxParticles) != TIXML_SUCCESS)
				ScThrowErr("xml preferences graphics element could not find maxParticles attribute");
			
			// bomb elements
			
			TiXmlElement *xBomb = docHandle.FirstChildElement("preferences").FirstChild("bomb").Element();
			if (!xBomb)
				ScThrowErr("xml preferences could not find bomb element");
			if (xBomb->QueryDoubleAttribute("time", &fuseLength) != TIXML_SUCCESS)
				ScThrowErr("xml preferences bomb element could not find time attribute");
			
			// music element
			
			TiXmlElement *xMusic = docHandle.FirstChildElement("preferences").FirstChild("music").Element();
			const char *music_attr = xMusic->Attribute("enabled");
			if (music_attr == NULL)
				ScThrowErr("xml preferences music element could not find enabled attribute");
			if (strcmp(music_attr, "yes") == 0)
				music = 1;
			else if (strcmp(music_attr, "no") == 0)
				music = 0;
			else
				ScThrowErr("xml preferences music attribute has unknown value");
			
			// sounds element
			
			TiXmlElement *xSounds = docHandle.FirstChildElement("preferences").FirstChild("sounds").Element();
			const char *sounds_attr = xSounds->Attribute("enabled");
			if (sounds_attr == NULL)
				ScThrowErr("xml preferences music element could not find enabled attribute");
			if (strcmp(sounds_attr, "yes") == 0)
				sounds = 1;
			else if (strcmp(sounds_attr, "no") == 0)
				sounds = 0;
			else
				ScThrowErr("xml preferences sounds attribute has unknown value");
		}
		catch (ScException e)
		{
			e.printMsg();
			printf("Preferences file is corrupt. Creating new preferences file...\n");
			defaultPrefs();
			savePreferences(filename);
		}
    }
    
    delete xmlDoc;
}

TcPreferences::~TcPreferences()
{
	free(filename);
}

void TcPreferences::defaultPrefs()
{
	screenwidth = 800;
	screenheight = 600;
	fullscreen = false;
	maxParticles = 6000;
	fuseLength = 60.0;
	music = true;
	sounds = true;
}

void TcPreferences::savePreferences(const char *filename)
{
	TiXmlDocument *xmlDoc = new TiXmlDocument(filename);
    TiXmlDeclaration decl("1.0", "UTF-8", "yes");
    TiXmlElement prefs("preferences");
    TiXmlElement screen("screen");
    TiXmlElement graphics("graphics");
    TiXmlElement bomb("bomb");
    TiXmlElement musicElem("music");
    TiXmlElement soundsElem("sounds");

    screen.SetAttribute("width", screenwidth);
    screen.SetAttribute("height", screenheight);
    screen.SetAttribute("fullscreen", fullscreen ? "yes" : "no");
    graphics.SetAttribute("maxParticles", maxParticles);
    bomb.SetDoubleAttribute("time", fuseLength);
    musicElem.SetAttribute("enabled", music ? "yes" : "no");
    soundsElem.SetAttribute("enabled", sounds ? "yes" : "no");

    prefs.InsertEndChild(screen);
    prefs.InsertEndChild(graphics);
    prefs.InsertEndChild(bomb);
    prefs.InsertEndChild(musicElem);
    prefs.InsertEndChild(soundsElem);
    xmlDoc->InsertEndChild(decl);
    xmlDoc->InsertEndChild(prefs);
    xmlDoc->SaveFile();
    delete xmlDoc;
}

void TcPreferences::drawPrefs()
{
    int menuResult;
	char screenDimsStr[256], fullscreenStr[256], maxParticlesStr[256], fuseLengthStr[256], musicStr[256], soundsStr[256];
	int screenwidthTemp = screenwidth, screenheightTemp = screenheight;
	bool fullscreenTemp = fullscreen;
	int maxParticlesTemp = maxParticles;
	double fuseLengthTemp = fuseLength;
	bool musicTemp = music;
	bool soundsTemp = sounds;
	char *items[] = {screenDimsStr, fullscreenStr, maxParticlesStr, fuseLengthStr, musicStr, soundsStr, "Cancel", "Save"};
	const int kPrefItems = 8;
	Menu menu(items, kPrefItems);
	
	do {
		sprintf(screenDimsStr, "Screen: %d x %d", screenwidthTemp, screenheightTemp);
		sprintf(fullscreenStr, "Fullscreen: %s", fullscreenTemp ? "yes" : "no");
		sprintf(maxParticlesStr, "Max Particles: %d", maxParticlesTemp);
		sprintf(fuseLengthStr, "Fuse Length: %.0lf secs", fuseLengthTemp);
		sprintf(musicStr, "Music: %s", musicTemp ? "yes" : "no");
		sprintf(soundsStr, "Sounds: %s", soundsTemp ? "yes" : "no");
		
        DrawSplash();
		menuResult = menu.doMenu();
		switch (menuResult)
		{
			case 0:
				if (screenwidthTemp == 800) {
					screenwidthTemp = 1024;
					screenheightTemp = 768;
				}
				else {
					screenwidthTemp = 800;
					screenheightTemp = 600;
				}
				break;
			case 1:
				fullscreenTemp = !fullscreenTemp;
				break;
			case 2:
				if (maxParticlesTemp < 10000)
					maxParticlesTemp += 1000;
				else
					maxParticlesTemp = 0;
				break;
			case 3:
				if (fuseLengthTemp < 180)
					fuseLengthTemp += 15;
				else
					fuseLengthTemp = 5;
				break;
				break;
			case 4:
				musicTemp = !musicTemp;
				break;
			case 5:
				soundsTemp = !soundsTemp;
				break;
			
			case 6:	// Cancel
				break;
			case 7:	// OK
				modifiedScreenPrefs = screenwidth != screenwidthTemp ||
									screenheight != screenheightTemp ||
									fullscreen != fullscreenTemp;
				screenwidth = screenwidthTemp;
				screenheight = screenheightTemp;
				fullscreen = fullscreenTemp;
				maxParticles = maxParticlesTemp;
				fuseLength = fuseLengthTemp;
				music = musicTemp;
				sounds = soundsTemp;
				savePreferences(filename);
				break;
			default:
				break;
		}
	} while (menuResult < kPrefItems - 2);
}
